#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
/*
     point and click xlib input routines.

     includes listboxes, textboxes and buttons

     last modified:

        12/2/95 added noedit to text boxes structure and check code
        12/2/95 removed the main win events from yes_no and info
        11/28/96 started writing xinput.doc
        12/29/96 changed to XwindowEvent in yes_no and info_window
        
        1/30/97 generalization started
        
        2/23/97 added YES NO DEFAULT defines for yes_no_window
        
        2/26/97 added default_button member and code for button
        	double width border 
        	added def (default) parameter to yes_no_window routine
        	  (much cleaner handling of <cr> default now)
        	  
	2/27/97 added click to exit text box input  
	
	4/14/97 added PASSWD choice for noedit      	  
        	  

*/

/* X include files */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#ifdef DVX
    #include <X11/cursorfo.h>
#else
    #include <X11/cursorfont.h>
#endif

#include "xinput.h"

int debug;

#define XI_EVENTS ButtonPressMask | ExposureMask | PointerMotionMask | KeyPressMask | LeaveWindowMask | StructureNotifyMask

/*****************************************************************

   Input Window in X routines: Text and list boxes and buttons


   includes info_window and yes_no_window routines

*****************************************************************/

xi_init(Display *display,int screen,char *fore_color,char *back_color,char *font_name)
{
/*
	call to set up X variables for xinput routines
	
	call with xi_init(display,screen,NULL,NULL,NULL) for defaults 
		(black on white with 8x13 font)
	
*/	
XColor color_exact,color_screen;
XFontStruct *font_info;

	if(display == NULL)
	{
		printf("xinput: null display\n"); 
		exit(-1);
	}
	xi_display = display;

        xi_colormap = DefaultColormap(xi_display,screen);

       /* default pixel values */
        xi_foreground = BlackPixel(xi_display,screen);
        xi_background = WhitePixel(xi_display,screen);
        
        if(back_color != NULL)
        {
        	if(debug)
        		printf("back pixel name = %s \n",back_color);
        		
        	if(XLookupColor(xi_display,xi_colormap,back_color,&color_exact,&color_screen) != 0)
        	{
        		XAllocColor(xi_display,xi_colormap,&color_screen);
        		xi_background = color_screen.pixel;
        		if(debug)
        			printf(" XlookupColor ok \n");
        	}
        }
        		
        
        if(debug)
        	printf("Background pixel = %i \n",xi_background);
        	
        if(fore_color != NULL)
        {
        	if(debug)
        		printf("foreground pixel name = %s \n",fore_color);
        		
        	if(XLookupColor(xi_display,xi_colormap,fore_color,&color_exact,&color_screen) != 0)
        	{
        		XAllocColor(xi_display,xi_colormap,&color_screen);
        		xi_foreground = color_screen.pixel;
        		if(debug)
        			printf(" XlookupColor ok \n");
        	}
        }
	
	if(font_name != NULL)
	{      
        	font_info = XLoadQueryFont(xi_display,font_name);
        }
        else
        {
        	font_info = XLoadQueryFont(xi_display,XI_FONT);
        }
        
        xi_font = font_info->fid;
        xi_fwidth = font_info->max_bounds.width;
        xi_fheight = font_info->max_bounds.ascent+font_info->max_bounds.descent;
        xi_fdescent = font_info->max_bounds.descent;
        
        if(debug)
        	printf("Font width is %i and height is %i \n",xi_fwidth,xi_fheight);

}


open_input_window(Window *window,GC *gc,int xpos,int ypos,
		int xsize,int ysize,char *title)
{

/*

        for input windows

*/


XClassHint *classhint = NULL;  /* needed for R4 call */
XWMHints *wmhint = NULL;
XTextProperty xtitle;
XSizeHints hints;

        /* default program-specified window position and size */
        hints.x=xpos;
        hints.y=ypos;

        hints.width=xsize;
        hints.height=ysize;
        
        if(debug)
        {
        	printf("open in win: at %i %i size = %ix%i \n",xpos,ypos,xsize,ysize);
        }

        /* window creation */
        *window = XCreateSimpleWindow(xi_display,
                DefaultRootWindow(xi_display),
                hints.x, hints.y, hints.width, hints.height,
                5, xi_foreground, xi_background);

        /* This uses R4 WM call */

        XStringListToTextProperty(&title,1,&xtitle);

        /* default program-specified window position and size */

        hints.flags = USPosition | USSize | PMaxSize | PMinSize ;

        hints.min_width = xsize;
        hints.min_height = ysize;
        hints.max_width = xsize;
        hints.max_height = ysize;

        XSetWMProperties(xi_display,*window,&xtitle,&xtitle,NULL,0,
                &hints,wmhint,classhint);

        /* GC creation and initialization */
        *gc = XCreateGC(xi_display, *window, 0, 0);
        XSetBackground(xi_display, *gc, xi_background);
        XSetForeground(xi_display, *gc, xi_foreground);
        XSetFont(xi_display,*gc,xi_font);

        /* input event selection */
        XSelectInput(xi_display,*window, XI_EVENTS );

        /* window mapping */
        XMapRaised(xi_display, *window);

}

redraw_inwin(Window xwin,GC gc,int win)
{
/*

        redraws the input window: buttons, text and list boxes

        clears the window first

*/

int i,ii;
char temp[10],hidden[100];

       XClearWindow(xi_display,xwin);

       /* draw text input boxes */

       if(inputwin[win].num_inputs > 0)
       {
        for(i=0;i<inputwin[win].num_inputs;i++)
        {
           if(inputwin[win].inputs[i].disable != 1)
           {
           	if(inputwin[win].inputs[i].noedit != PASSWD)
           	{
                     draw_input(xwin,gc,inputwin[win].inputs[i].row,
                        inputwin[win].inputs[i].col,
                        inputwin[win].inputs[i].text,
                        inputwin[win].inputs[i].maxlen,
                        inputwin[win].inputs[i].caption);
                 }
                 else
           	 {
           	     strcpy(hidden,inputwin[win].inputs[i].text);
           	     for(ii=0;ii<strlen(hidden);ii++)
           	     	hidden[ii]='*';	
                     draw_input(xwin,gc,inputwin[win].inputs[i].row,
                        inputwin[win].inputs[i].col,
                        hidden,
                        inputwin[win].inputs[i].maxlen,
                        inputwin[win].inputs[i].caption);
                 }
                 
           }
        }
       }
        /* draw list boxes */

       if(inputwin[win].num_lists > 0)
       {
        for(i=0;i<inputwin[win].num_lists;i++)
        {
           if(inputwin[win].lists[i].disable != 1)
           {
                if(inputwin[win].lists[i].numeric == 0)
                {
                   draw_input(xwin,gc,inputwin[win].lists[i].row,
                        inputwin[win].lists[i].col,
                        inputwin[win].lists[i].text[inputwin[win].lists[i].item],
                        inputwin[win].lists[i].maxlen,
                        inputwin[win].lists[i].caption);
                }
                if(inputwin[win].lists[i].numeric == 1) /* count starting at 1 */
                {
                  sprintf(temp,"%2i",inputwin[win].lists[i].item+1);
                  draw_input(xwin,gc,inputwin[win].lists[i].row,
                        inputwin[win].lists[i].col,
                        temp,
                        inputwin[win].lists[i].maxlen,
                        inputwin[win].lists[i].caption);
                }

                if(inputwin[win].lists[i].numeric == 2) /* count starting at 0 */
                {
                  sprintf(temp,"%2i",inputwin[win].lists[i].item);
                  draw_input(xwin,gc,inputwin[win].lists[i].row,
                        inputwin[win].lists[i].col,
                        temp,
                        inputwin[win].lists[i].maxlen,
                        inputwin[win].lists[i].caption);
                }

              draw_list_arrows(xwin,gc,win,i);
           }
        }
       }
        draw_buttons(xwin,gc,win);
}


close_input_window(Window window,GC gc)
{
/*
        closes the input window and frees the GC

*/
        XFreeGC(xi_display, gc);
        XDestroyWindow(xi_display, window);
}

/*******************************************************

        information window routines

*****************************************************/

info_window(int xpos,int ypos,char *text)
{
/*

                opens a second window for input
                
                window is centered on xpos,ypos

                includes event loop with redraw and resize

                needs the redraw for the main window added.

                returns nothing

*/

int done,i,in,q,len,w2xsize,w2ysize;
int col,row;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;
long my2event_mask;
int xpos2,ypos2;

        len=strlen(text);
        w2xsize = len*xi_fwidth + 50;
        if(w2xsize < (BUTTONWIDTH+20)) w2xsize = BUTTONWIDTH + 20;
        w2ysize = 70;
        
        /* center window on given coords */
        xpos2=xpos-w2xsize/2;
        if(xpos2 < 0) xpos2 = 0;
        ypos2 = ypos - w2ysize/2;
        if(ypos2 < 0) ypos2 = 0;

        open_input_window(&my2window,&my2gc,xpos2,ypos2,w2xsize,w2ysize,"Information");

	my2event_mask = XI_EVENTS ;
	
        /* event-reading loop to get input  */
        done = 0;
        while( done == 0) {
            /* read the next event */
            /*
            XNextEvent(xi_display, &my2event);
            */
            XWindowEvent(xi_display, my2window, my2event_mask, &my2event);
            /* events for input window */
            if(my2event.xany.window == my2window)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                                    redraw_info(my2window,my2gc,text,w2xsize);

                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                             XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:

                                    col=my2event.xmotion.x;
                                    row=my2event.xmotion.y;
                                    done = 1;
                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &xi_key, 0);
                                if( (xi_key == XK_Escape )||(xi_key == XK_Return) || (xi_key == XK_KP_Enter) || (xi_key == XK_Linefeed))
                                {
                                     done = 1;
                                }
                        break;
                } /* switch(my2event.type) */
             } /* if my window */
        } /* while(done == 0) */

        close_input_window(my2window,my2gc);
        XFlush(xi_display);

        return(done);
}

redraw_info(Window xwin,GC gc,char *text,int w2xsize)
{
int len,row,col;

        len=strlen(text);

        row=20;
        col = 25;

        XDrawImageString( xi_display,xwin, gc,col,row,text,len);

        col = w2xsize/2;
        row = 50;

        draw_button(xwin,gc,row,col,BUTTONWIDTH,BUTTONHEIGHT,"Dismiss",YES);

}

/*******************************************************

        yes/no window routines

*****************************************************/


int yes_no_window(int xpos,int ypos,char *text,int def)
{
/*
	call with:
		xpos,ypos = window center position
		text = Question
		def = YES or NO, <cr> default answer

        opens a second window for input
                
        includes event loop with redraw and resize

        needs the redraw for the main window added.

        returns 1 == YES 
                0 == NO based on click
                value of def if return pressed (def)
                	 
                (use defines YES NO)

*/

int done,i,in,q,len;
int x,y,w2xsize,w2ysize;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;
long my2event_mask;
int xpos2,ypos2;

        len=strlen(text);
        w2xsize = len*xi_fwidth + 50;
        if(w2xsize < (2*BUTTONWIDTH+20)) w2xsize = 2*BUTTONWIDTH + 20;
        w2ysize = 70;
        
        /* center window on given coords */
        xpos2=xpos-w2xsize/2;
        if(xpos2 < 0) xpos2 = 0;
        ypos2 = ypos - w2ysize/2;
        if(ypos2 < 0) ypos2 = 0;

        open_input_window(&my2window,&my2gc,xpos2,ypos2,w2xsize,w2ysize,"Yes or No?");

	my2event_mask = XI_EVENTS ;
	
        /* event-reading loop to get input  */
        done = -1;
        while( done == -1) {
                /* read the next event */
                /*
            XNextEvent(xi_display, &my2event);
            */
            /* events for input window */
            
            XWindowEvent(xi_display, my2window, my2event_mask, &my2event);

            if(my2event.xany.window == my2window)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                                        redraw_yesno(my2window,my2gc,text,w2xsize,def);

                        break;


                        /* process keyboard mapping changes */
                        case MappingNotify:
                             XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:

                                    x=my2event.xmotion.x;
                                    y=my2event.xmotion.y;
                                    if((q = yes_or_no(my2window,my2gc,x,y,w2xsize)) > -1) done = q;

                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &xi_key, 0);
                                if( (xi_key == XK_Return) || (xi_key == XK_KP_Enter) || (xi_key == XK_Linefeed))
                                {
                                     done = DEFAULT; /* default value */
                                }
                        break;
                } /* switch(my2event.type) */
             } /* if my window */
        } /* while(done == -1) */

        close_input_window(my2window,my2gc);
        XFlush(xi_display);

	if(done != DEFAULT)
	        return(done);
	else
		return(def);
}

int yes_or_no(Window xwin,GC gc,int x,int y,int w2xsize)
{
int x1,x2,y1,y2,row,col,l;

        /* yes button */
        col = w2xsize/2 - BUTTONWIDTH/2 - 5;
        row = 50;

        x1=col - BUTTONWIDTH/2;
        y1=row - BUTTONHEIGHT/2;
        x2=col + BUTTONWIDTH/2;
        y2=row + BUTTONHEIGHT/2;
        if(x >= x1 && x <= x2 && y>=y1 && y<= y2 )
        {
                XFillRectangle(xi_display,xwin,gc,
                        x1,y1,BUTTONWIDTH,BUTTONHEIGHT);

                XSetForeground(xi_display,gc,xi_background);
                XSetBackground(xi_display,gc,xi_foreground);

                l=3;
                x1=col-(xi_fwidth*l)/2;

                XDrawImageString(xi_display,xwin,gc,x1,row,"yes",l);

                XSetForeground(xi_display,gc,xi_foreground);
                XSetBackground(xi_display,gc,xi_background);

                XFlush(xi_display);

                return(YES);
        }

        /* no button */
        col = w2xsize/2 + BUTTONWIDTH/2 + 5;
        row = 50;

        x1=col - BUTTONWIDTH/2;
        y1=row - BUTTONHEIGHT/2;
        x2=col + BUTTONWIDTH/2;
        y2=row + BUTTONHEIGHT/2;
        if(x >= x1 && x <= x2 && y>=y1 && y<= y2 )
        {
                XFillRectangle(xi_display,xwin,gc,
                        x1,y1,BUTTONWIDTH,BUTTONHEIGHT);

                XSetForeground(xi_display,gc,xi_background);
                XSetBackground(xi_display,gc,xi_foreground);

                l=2;
                x1=col-(xi_fwidth*l)/2;

                XDrawImageString(xi_display,xwin,gc,x1,row,"no",l);

                XSetForeground(xi_display,gc,xi_foreground);
                XSetBackground(xi_display,gc,xi_background);

                XFlush(xi_display);

                return(NO);
        }

        return(-1);
}

redraw_yesno(Window xwin,GC gc,char *text,int w2xsize,int def)
{
int len,row,col;

        len=strlen(text);

        row=20;
        col = 25;

        XDrawImageString( xi_display,xwin, gc,col,row,text,len);

        /* yes button */
        col = w2xsize/2 - BUTTONWIDTH/2 - 5;
        row = 50;

	if(def == YES)
        	draw_button(xwin,gc,row,col,BUTTONWIDTH,BUTTONHEIGHT,"Yes",YES);
        else
        	draw_button(xwin,gc,row,col,BUTTONWIDTH,BUTTONHEIGHT,"Yes",NO);

        /* no button */
        col = w2xsize/2 + BUTTONWIDTH/2 + 5;
        row = 50;

	if(def == NO)	
        	draw_button(xwin,gc,row,col,BUTTONWIDTH,BUTTONHEIGHT,"No",YES);
        else
        	draw_button(xwin,gc,row,col,BUTTONWIDTH,BUTTONHEIGHT,"No",NO);

}

/*******************************************************

        input text box window routines

*****************************************************/


int check_input(int win,int row,int col)
{
/*
        checks for mouse click in a text input region,

        returns the region or -1 = none

*/
int cent_row,cent_col,d_row,d_col,in;

        if(inputwin[win].num_inputs > 0)
        {
        for(in=0;in<inputwin[win].num_inputs;in++)
        {
           if((inputwin[win].inputs[in].disable != YES)&&(inputwin[win].inputs[in].noedit != YES))
           {
               cent_row = inputwin[win].inputs[in].row - .75*xi_fheight;
               cent_col = inputwin[win].inputs[in].col
                           + (xi_fwidth*(inputwin[win].inputs[in].maxlen+1))/2;

               d_row = abs(row - cent_row);
               d_col = abs(col-cent_col);

               if( (d_row <= (.75*xi_fheight) )&&
                   ( d_col <= ((inputwin[win].inputs[in].maxlen+1)*xi_fwidth)/2 ))
               {
                       return(in);
               }
           }
        }
        }

        return(-1);

}

int get_input(Window xwin,GC gc,int in, int win)
{
int done,i,k,row,col,maxlen;
char txt[20],input[100],caption[50];

int in_char,in_len;
XEvent my2event;

/*
        pass the default string in first
        row and col are the input location in the window (lowerleft)

        returns an input string allows editing in place

        keys supported: return, backspace, delete, left arrow, right arrow
                        escape, end, home  and all printable ascii

        always is in insert mode, no overstrike supported

        returns number of characters entered (including 0)

        returns -1 if escape pressed

        cursor changes to pencil for input
        
        click outside the textbox accepts the changes 
           and ends the edit, click is passed back to calling loop.

        includes event loop with redraw and resize

        need to add xterm like paste selection
*/

        maxlen = inputwin[win].inputs[in].maxlen;
        row = inputwin[win].inputs[in].row;
        col = inputwin[win].inputs[in].col;
        strcpy(input,inputwin[win].inputs[in].text);
        strcpy(caption,inputwin[win].inputs[in].caption);

        in_len = strlen(input);
        in_char = in_len;



        /* set cursor to pencil  */
        xi_cursor = XCreateFontCursor(xi_display,XC_pencil);
        XDefineCursor(xi_display,xwin,xi_cursor);

        redraw_input(xwin,gc,row,col,input,maxlen,in_char,inputwin[win].inputs[in].noedit);

        /* event-reading loop to get cursor position */
        done = 0;
        while( done == 0) {
            /* read the next event */
            XNextEvent(xi_display, &my2event);

            /* events for input window */
            if(my2event.xany.window == xwin)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                              {
                                     redraw_inwin(xwin,gc,win);
                                     redraw_input(xwin,gc,row,col,input,maxlen,in_char,inputwin[win].inputs[in].noedit);
                              }
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                                XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses to set edit position 
                           or exit if outside the input box */
                        case ButtonPress:
                        	if(check_input(win,my2event.xbutton.y,my2event.xbutton.x) == in)
                        	{
                                    in_char = (my2event.xmotion.x-col)/xi_fwidth;
                                    if(in_char < 0 ) in_char = 0;
                                    if(in_char > in_len) in_char = in_len;
                                    redraw_input(xwin,gc,row,col,input,maxlen,in_char,inputwin[win].inputs[in].noedit);
                                }
                                else
                                {
                                /*  uncomment to let exit click also go thru to calling window
                              	    XPutBackEvent(xi_display,&my2event);
                              	    */
                              	    done = strlen(input);
                                    strcpy(inputwin[win].inputs[in].text,input);
                                    XUndefineCursor(xi_display,xwin);
                                    draw_input(xwin,gc,row,col,input,maxlen,caption);
                                    return(done);
                              	}
                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &xi_key, 0);

                                if( xi_key == XK_Escape )
                                {
                                    XUndefineCursor(xi_display,xwin);
                                    draw_input(xwin,gc,row,col,input,maxlen,caption);
                                    return(-1);
                                }

                                if((xi_key == XK_Return) || (xi_key == XK_KP_Enter) || (xi_key == XK_Linefeed))
                                {
                                    done = strlen(input);
                                    strcpy(inputwin[win].inputs[in].text,input);
                                    XUndefineCursor(xi_display,xwin);
                                    if(inputwin[win].inputs[in].noedit == PASSWD)
                                    {
           	     			for(i=0;i<strlen(input);i++)
           	     				input[i]='*';	
           	     		    }

                                    draw_input(xwin,gc,row,col,input,maxlen,caption);
                                    return(done);
                                }

                                if( (xi_key == XK_BackSpace) || (xi_key == XK_Delete))
                                {
                                    if(in_len > 0 )
                                    {
                                         remove_char(input,in_char,in_len);
                                         in_len--;
                                         if(in_char > 0) in_char--;
                                         redraw_input(xwin,gc,row,col,input,maxlen,in_char,inputwin[win].inputs[in].noedit);
                                    }
                                }

                                if( xi_key == XK_Right)
                                {
                                   if(in_char < in_len) in_char++;
                                }

                                if( xi_key == XK_Left)
                                {
                                   if(in_char > 0) in_char--;
                                }

                                if( xi_key == XK_Home)
                                {
                                   in_char=0;
                                }

                                if( xi_key == XK_End)
                                {
                                   in_char=in_len;
                                }

                                if(((xi_key >= XK_KP_Space)&&(xi_key <= XK_KP_9)||(xi_key >= XK_space)&&(xi_key <= XK_asciitilde)))
                                {
                                    if(i > 0)
                                    {
                                          for(k=0;k<i;k++)
                                          {
                                             if(in_len < maxlen )
                                             {
                                                insert_char(input,in_char,in_len,txt[k]);
                                                in_len++;
                                                in_char++;
                                             }
                                          }
                                    }
                                }

                                redraw_input(xwin,gc,row,col,input,maxlen,in_char,inputwin[win].inputs[in].noedit);

                        break;
                } /* switch(my2event.type) */
            } /* window if */
        } /* while(done == 0) */

        /* set cursor back */

        XUndefineCursor(xi_display,xwin);

        return(done);
}

ftoa(char *result,char *fmt,float value)
{
char temp[20];

        sprintf(temp,fmt,value);

        while( temp[0] == ' ' )
        {
                remove_char(temp,1,strlen(temp));
        }
        strcpy(result,temp);
}

remove_char(char *input,int at,int len)
{
/*
        removes a character at 'at' from a string
*/
int i;
        if( at == len)
        {
                input[at-1]='\0';
        }
        else
        {
                for(i=at-1;i<len;i++) input[i]=input[i+1];
        }

}

insert_char(char *input,int at,int len,char c)
{
/*
        inserts a character at 'at' in a string
*/
int i;
        if( at == len)
        {
                input[at]=c;
                input[at+1]='\0';
        }
        else
        {
               for(i=len;i>=at;i--) input[i+1]=input[i];
               input[at]=c;
        }
}

redraw_input(Window xwin,GC gc,int row,int col,char *input,int maxlen,int at,int noedit)
{
/*
        redraw an input region for a list or text (input) box

        positions the reverse video cursor at 'at'

        draws the border with a double line width
        
        if noedit == PASSWD shows '*' instead of text

*/
int l,i;
char blank[100],temp[10];
        l = strlen(input);
        for(i=0;i<=maxlen;i++)
        {
           blank[i]=' ';
        }
        blank[maxlen]='\0';

        XSetBackground(xi_display, gc, xi_background);
        XSetForeground(xi_display, gc, xi_foreground);

        XDrawImageString( xi_display,xwin, gc,col,row,blank,maxlen);

       draw_input_box(xwin,gc,row, col, maxlen,2);

       temp[1]='\0';

       if(l > 0)
       {
            for(i=0;i<l;i++)
            {
                 if( i == at )
                 {
                     XSetBackground(xi_display, gc, xi_foreground);
                     XSetForeground(xi_display, gc, xi_background);
                 }
                 if( i == (at+1) )
                 {
                     XSetBackground(xi_display, gc, xi_background);
                     XSetForeground(xi_display, gc, xi_foreground);
                 }

		 if(noedit != PASSWD)
                 	temp[0]=input[i];
                 else
                 	temp[0]='*';
                 	
                 XDrawImageString( xi_display,xwin, gc,col+i*xi_fwidth,row,temp,1);
            }
       }
       if((l == 0 ) || (at == l))
       {
            XSetBackground(xi_display, gc, xi_foreground);
            XSetForeground(xi_display, gc, xi_background);
            temp[0]=' ';
            XDrawImageString( xi_display,xwin,  gc,col+l*xi_fwidth,row,temp,1);
       }

       XSetBackground(xi_display, gc, xi_background);
       XSetForeground(xi_display, gc, xi_foreground);

}

draw_input(Window xwin,GC gc,int row,int col,char *input,int maxlen,char *caption)
{
/*

        draws an input/list box with a single line width border

        first erases a double line border and the maxlen of text


*/
int l,i,cc;
char blank[100];
        l = strlen(input);
        for(i=0;i<maxlen;i++)
        {
           blank[i]=' ';
        }
        blank[maxlen]='\0';

        XSetForeground(xi_display, gc, xi_background);
        draw_input_box(xwin,gc,row, col, maxlen,2);

        XSetBackground(xi_display, gc, xi_background);
        XSetForeground(xi_display, gc, xi_foreground);

        XDrawImageString( xi_display,xwin, gc,col,row,blank,maxlen);

        XDrawImageString( xi_display,xwin, gc,col,row,input,l);

        draw_input_box(xwin,gc,row, col, maxlen,1);

        l = strlen(caption);
        cc = col - (l+2)*xi_fwidth;
        XDrawImageString( xi_display,xwin, gc,cc,row,caption,l);

}

draw_input_box(Window xwin,GC gc,int row,int col,int maxlen,int line)
{
/*
        draws an input box for text/list input of thickness line

*/
int xx1,xx2,yy1,yy2;
XGCValues gcv;

             XGetGCValues(xi_display,gc,GCLineWidth,&gcv);

             if(line == 2)
             {
                        gcv.line_width = 2;
                        XChangeGC(xi_display, gc, GCLineWidth, &gcv);
             }
             else
             {
                        gcv.line_width = 1;
                        XChangeGC(xi_display, gc, GCLineWidth, &gcv);
             }

            xx1=col - xi_fwidth/2;
            yy1=row + xi_fheight/2;
            xx2=xx1 + (maxlen+1)*xi_fwidth + xi_fwidth/2;
            yy2=yy1;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=col - xi_fwidth/2;
            yy1=row - xi_fheight - xi_fheight/2;
            xx2=xx1 + (maxlen+1)*xi_fwidth + xi_fwidth/2;
            yy2=yy1;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=col - xi_fwidth/2;
            yy1=row + xi_fheight/2;
            xx2=xx1;
            yy2=row - xi_fheight - xi_fheight/2;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=col + (maxlen+1)*xi_fwidth ;
            yy1=row + xi_fheight/2;
            xx2=xx1;
            yy2=row - xi_fheight - xi_fheight/2;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            gcv.line_width = 1;
            XChangeGC(xi_display, gc, GCLineWidth, &gcv);

}

/*******************************************************

        list box window routines

*****************************************************/


int check_list(int win,int row,int col)
{
/*
        takes care of up/down arrows clicks

        returns the region number or -1 = none

*/
int cent_row,cent_col,d_row,d_col,in;

       if(inputwin[win].num_lists > 0)
       {
        for(in=0;in<inputwin[win].num_lists;in++)
        {
           if(inputwin[win].lists[in].disable != 1)
           {

             /* check for arrows click */

             cent_col=inputwin[win].lists[in].col
                 + xi_fwidth*(inputwin[win].lists[in].maxlen+2);

             cent_row=inputwin[win].lists[in].row - xi_fheight/2;

             d_row = row - cent_row;
             d_col = abs(col - cent_col);

             /* up arrow */
             if( (d_row < 0) &&(d_row > -((3*xi_fheight)/2))&&(d_col < xi_fwidth/2))
             {
                 if(inputwin[win].lists[in].item > 0)
                 {
                     inputwin[win].lists[in].item --;
                     return(in);
                 }
             }
             /* down arrow */
             if( (d_row > 0) &&(d_row < ((3*xi_fheight)/2))&&(d_col < xi_fwidth/2))
             {
                 if(inputwin[win].lists[in].item < (inputwin[win].lists[in].list_len -1))
                 {
                     inputwin[win].lists[in].item ++;
                     return(in);
                 }
             }
           }
        }
       }
       return(-1);
}


draw_list_arrows(Window xwin,GC gc,int win,int list)
{
int x_c,y_c;
int xx1,xx2,yy1,yy2;

        x_c=inputwin[win].lists[list].col
            + xi_fwidth*(inputwin[win].lists[list].maxlen+2);

        y_c=inputwin[win].lists[list].row - xi_fheight/2;

        /* up arrow */
        if(inputwin[win].lists[list].item > 0)
        {
            xx1=x_c - xi_fwidth/2;
            yy1=y_c - xi_fheight/4;
            xx2=x_c;
            yy2=y_c - (3*xi_fheight)/2;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=x_c + xi_fwidth/2;
            yy1=y_c - xi_fheight/4;
            xx2=x_c;
            yy2=y_c - (3*xi_fheight)/2;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=x_c + xi_fwidth/2;
            yy1=y_c - xi_fheight/4;
            xx2=x_c - xi_fwidth/2;
            yy2=y_c - xi_fheight/4;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);
        }

        /* down arrow */
        if(inputwin[win].lists[list].item < inputwin[win].lists[list].list_len - 1)
        {
            xx1=x_c - xi_fwidth/2;
            yy1=y_c + xi_fheight/4;
            xx2=x_c;
            yy2=y_c + (3*xi_fheight)/2;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=x_c + xi_fwidth/2;
            yy1=y_c + xi_fheight/4;
            xx2=x_c;
            yy2=y_c + (3*xi_fheight)/2;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);

            xx1=x_c + xi_fwidth/2;
            yy1=y_c + xi_fheight/4;
            xx2=x_c - xi_fwidth/2;
            yy2=y_c + xi_fheight/4;
            XDrawLine(xi_display,xwin,gc,xx1,yy1,xx2,yy2);
        }

}


/***************************************************************

        window routines for buttons: for inputwin routines

***************************************************************/


draw_buttons(Window xwin,GC gc,int win)
{
int i,x,y,l;
char txt[50];

/*
             draws all the buttons

             assumes menu1

             and inputwin[win].buttons[i] structure

*/

       if(inputwin[win].num_buttons > 0)
       {
        for(i=0;i<inputwin[win].num_buttons;i++)
        {
           if(inputwin[win].buttons[i].disable != 1)
           {
           	if(inputwin[win].default_button == i)
           	{
                      draw_button(xwin,gc,inputwin[win].buttons[i].row,
                            inputwin[win].buttons[i].col,
                            inputwin[win].button_width,
                            inputwin[win].button_height,
                            inputwin[win].buttons[i].text,YES);
                 }
                 else
           	 {
                      draw_button(xwin,gc,inputwin[win].buttons[i].row,
                            inputwin[win].buttons[i].col,
                            inputwin[win].button_width,
                            inputwin[win].button_height,
                            inputwin[win].buttons[i].text,NO);
                 }
           }
         }
       }

}

draw_button(Window xwin,GC gc,int y,int x,int width,int height,char *text,int highlight)
{
int l,x1,y1;
                l=strlen(text);
                x1=x-(xi_fwidth*l)/2;

                XDrawImageString( xi_display,xwin, gc,x1,y,text,l);

                x1=x-width/2;
                y1=y-height/2;

                XDrawRectangle(xi_display,xwin,gc,x1,y1,width,height);
                
                if(highlight == YES)
                {
                	XDrawRectangle(xi_display,xwin,gc,x1-1,y1-1,width+2,height+2);
                }
}


int check_button(Window xwin,GC gc,int x,int y,int win)
{
/*
      returns the number of the button pressed

      using the position on the screen to determine which is pressed

      assumes menu1 and buttons[]

*/


int x1,y1,i,l;

        i=which_button(x,y,win);

        if(i > -1)
        {
                /* reverse video the button picked */

                        x1=inputwin[win].buttons[i].col-((int)inputwin[win].button_width)/2;
                        y1=inputwin[win].buttons[i].row-((int)inputwin[win].button_height)/2;

                        XFillRectangle(xi_display,xwin,gc,
                                x1,y1,inputwin[win].button_width,inputwin[win].button_height);

                        XSetForeground(xi_display,gc,xi_background);
                        XSetBackground(xi_display,gc,xi_foreground);

                        l=strlen(inputwin[win].buttons[i].text);
                        x1=inputwin[win].buttons[i].col-(xi_fwidth*l)/2;

                        XDrawImageString( xi_display,
                                xwin, gc,x1, inputwin[win].buttons[i].row,
                                inputwin[win].buttons[i].text, l);

                        XSetForeground(xi_display,gc,xi_foreground);
                        XSetBackground(xi_display,gc,xi_background);

                        XFlush(xi_display);
         }

         return(i);
}

int which_button(int x,int y,int win)
{
int x1,x2,y1,y2,i;

       if(inputwin[win].num_buttons > 0)
       {
        for(i=0;i<inputwin[win].num_buttons;i++)
        {
           if(inputwin[win].buttons[i].disable != 1)
           {
                x1=inputwin[win].buttons[i].col-(int)inputwin[win].button_width/2;
                y1=inputwin[win].buttons[i].row-(int)inputwin[win].button_height/2;
                x2=inputwin[win].buttons[i].col+inputwin[win].button_width/2;
                y2=inputwin[win].buttons[i].row+inputwin[win].button_height/2;
                if(x >= x1 && x <= x2 && y>=y1 && y<= y2 )
                {
                        return(i);
                }
           }
        }
       }
        return(-1);
}

